//-----------------------------------------------------------------------------------
//         FILE FOR GENERATION OF RANDOM NUMBERS AND EVALUATION OF DENSITIES 
//                               by GERNOT MUELLER, 2004
//-----------------------------------------------------------------------------------

#ifndef __RANDOM_H
#define __RANDOM_H

#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <time.h>
#include "shared2.h"
#include "matrixGM.h"


//-----------------------------------------------------------------------------------
//                     INITIALIZATION OF THE RANDOM NUMBER GENERATOR
//-----------------------------------------------------------------------------------
// set the pseudo random number generator on a random starting position

void randomize(void);

//-----------------------------------------------------------------------------------
//               GENERATION OF RANDOM NUMBERS FROM UNIVARIATE DISTRIBUTIONS
//-----------------------------------------------------------------------------------
// draw a random number from the Unif(a,b) distribution

double uniform(double a=0.00000, double b=1.00000);

//-----------------------------------------------------------------------------------
// draw a random number from the N(mu,sigma) distribution
// sigma = standard deviation!

double normal(const double & mu=0, const double & sigma=1);

//-----------------------------------------------------------------------------------
// draw a random number from the Exp(lambda) distribution

double rand_expo(double lambda);

//-----------------------------------------------------------------------------------
// draw a random number from the Gamma(a,b) distribution

double rand_gamma(double a,double b);
  
//-----------------------------------------------------------------------------------
// draw a random number double-side truncated normal distribution

double ds_trunc_normal(const double & left, const double & right,
                              const double & mu=0, const double & sigma=1);

//-----------------------------------------------------------------------------------
// draw a random number left-side truncated normal distribution

double ls_trunc_normal(const double & left,
                              const double & mu=0, const double & sigma=1);

//-----------------------------------------------------------------------------------
// draw a random number from right-side truncated normal distribution

double rs_trunc_normal(const double & right,
                              const double & mu=0, const double & sigma=1);
			      
//-----------------------------------------------------------------------------------
//                         DENSITIES FROM UNIVARIATE DISTRIBUTIONS
//-----------------------------------------------------------------------------------
// density of normal distribution

double dnorm(double x, double mu, double sigma);

// unnormalized density of gamma distribution
double undgamma(double y, double alpha, double beta);

//-----------------------------------------------------------------------------------
// density of student t-distribution
// definition see Gelman, Carlin, Stern, and Rubin: Bayesian Data Analysis
// nu: degress of freedom, mean: location parameter, sigma: scale parameter



// compute Gamma((nu+1)/2)/Gamma(nu/2)
double gammafactorsforstudent(unsigned int nu);

// store the factors Gamma((nu+1)/2)/Gamma(nu/2)
void savegammafactors(void);

// density of student t-distribution at y
double dstudent(double y, unsigned int nu, double mean=0.0000000000, 
                       double sigma=1.0000000000);

//-----------------------------------------------------------------------------------
// unnormalized density of discretized gamma distribution truncated at 1.5 and 128.5
// using Newton-Cotes with 5 nodes, evaluated at y

double unddisgamma(unsigned int y, double alpha, double beta);

//-----------------------------------------------------------------------------------
//                              MULTIVARIATE DISTRIBUTIONS
//-----------------------------------------------------------------------------------
// random numbers from multivariate normal-distribution 

Matrix mvnormal(Matrix & Mu, Matrix & Sigma);

//-----------------------------------------------------------------------------------
// unnormalized density of multivariate normal distribution, evaluated at X

double undmvnormal(Matrix & X, Matrix & Mu, Matrix & Sigma);
#endif
